/*
    TreeP Run Time Support
    Copyright (C) 2008-2012 Frank Sinapsi

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../trp/trp.h"
#include "./trpmysql.h"
#include <mysql.h>

typedef struct {
    uns8b tipo;
    void *s;
} trp_mysql_t;

static uns8b trp_mysql_print( trp_print_t *p, trp_mysql_t *obj );
static uns8b trp_mysql_close( trp_mysql_t *obj );
static uns8b trp_mysql_close_basic( uns8b flags, trp_mysql_t *obj );
static void trp_mysql_finalize( void *obj, void *data );

uns8b trp_mysql_init()
{
    extern uns8bfun_t _trp_print_fun[];
    extern uns8bfun_t _trp_close_fun[];

    _trp_print_fun[ TRP_MYSQL ] = trp_mysql_print;
    _trp_close_fun[ TRP_MYSQL ] = trp_mysql_close;
    return 0;
}

static uns8b trp_mysql_print( trp_print_t *p, trp_mysql_t *obj )
{
    if ( trp_print_char_star( p, "#mysql db" ) )
        return 1;
    if ( obj->s == NULL )
        if ( trp_print_char_star( p, " (closed)" ) )
            return 1;
    return trp_print_char( p, '#' );
}

static uns8b trp_mysql_close( trp_mysql_t *obj )
{
    return trp_mysql_close_basic( 1, obj );
}

static uns8b trp_mysql_close_basic( uns8b flags, trp_mysql_t *obj )
{
    uns8b res = 0;

    if ( obj->s ) {
        obj->s = NULL;
        trp_gc_remove_finalizer( (trp_obj_t *)obj );
    }
    return res;
}

static void trp_mysql_finalize( void *obj, void *data )
{
    trp_mysql_close_basic( 0, (trp_mysql_t *)obj );
}



