/*
    TreeP Run Time Support
    Copyright (C) 2008-2012 Frank Sinapsi

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "./trppix_internal.h"
#include <png.h>
#include <zlib.h>

#ifndef PNG_BYTES_TO_CHECK
#define PNG_BYTES_TO_CHECK 8
#endif
#ifndef png_jmpbuf
#define png_jmpbuf(png_ptr) ((png_ptr)->jmpbuf)
#endif

uns8b trp_pix_info_png( uns8b *cpath, uns32b *w, uns32b *h )
{
    FILE *fp = trp_fopen( cpath, "rb" );
    png_structp png_ptr;
    png_infop info_ptr;
    uns8b buf[ PNG_BYTES_TO_CHECK ];

    if ( fp == NULL )
        return 1;
    if ( fread( buf, 1, PNG_BYTES_TO_CHECK, fp ) != PNG_BYTES_TO_CHECK ) {
        fclose( fp );
        return 1;
    }
    if ( png_sig_cmp( buf, (png_size_t)0, PNG_BYTES_TO_CHECK ) ) {
        fclose( fp );
        return 1;
    }
    if ( ( png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING,
                                             NULL, NULL, NULL ) ) == NULL ) {
        fclose( fp );
        return 1;
    }
    if ( ( info_ptr = png_create_info_struct( png_ptr ) ) == NULL ) {
        png_destroy_read_struct( &png_ptr, NULL, NULL );
        fclose( fp );
        return 1;
    }
    if ( setjmp( png_jmpbuf( png_ptr ) ) ) {
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
        fclose( fp );
        return 1;
    }
    png_init_io( png_ptr, fp );
    png_set_sig_bytes( png_ptr, PNG_BYTES_TO_CHECK );
    png_read_info( png_ptr, info_ptr );
    *w = png_get_image_width( png_ptr, info_ptr );
    *h = png_get_image_height( png_ptr, info_ptr );
    png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
    fclose( fp );
    return 0;
}

uns8b trp_pix_load_png( uns8b *cpath, uns32b *w, uns32b *h, uns8b **data )
{
    int res = 1;
    FILE *fp = trp_fopen( cpath, "rb" );
    uns32b color_type, bpl, i;
    png_structp png_ptr;
    png_infop info_ptr;
    png_bytep *rows = NULL;
    uns8b buf[ PNG_BYTES_TO_CHECK ];

    if ( fp == NULL )
        return 1;
    if ( fread( buf, 1, PNG_BYTES_TO_CHECK, fp ) != PNG_BYTES_TO_CHECK ) {
        fclose( fp );
        return 1;
    }
    if ( png_sig_cmp( buf, (png_size_t)0, PNG_BYTES_TO_CHECK ) ) {
        fclose( fp );
        return 1;
    }
    if ( ( png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING,
                                             NULL, NULL, NULL ) ) == NULL ) {
        fclose( fp );
        return 1;
    }
    if ( ( info_ptr = png_create_info_struct( png_ptr ) ) == NULL ) {
        png_destroy_read_struct( &png_ptr, NULL, NULL );
        fclose( fp );
        return 1;
    }
    *data = NULL;
    if ( setjmp( png_jmpbuf( png_ptr ) ) ) {
        free( (void *)rows );
        free( (void *)( *data ) );
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
        fclose( fp );
        return 1;
    }
    png_init_io( png_ptr, fp );
    png_set_sig_bytes( png_ptr, PNG_BYTES_TO_CHECK );
    png_read_info( png_ptr, info_ptr );
    if ( png_get_bit_depth( png_ptr, info_ptr ) == 16 )
        png_set_strip_16( png_ptr );
    color_type = png_get_color_type( png_ptr, info_ptr );
    switch ( color_type & ~PNG_COLOR_MASK_ALPHA ) {
    case PNG_COLOR_TYPE_RGB:
        break;
    case PNG_COLOR_TYPE_PALETTE:
        png_set_palette_to_rgb( png_ptr );
        break;
    case PNG_COLOR_TYPE_GRAY:
        png_set_expand_gray_1_2_4_to_8( png_ptr );
        png_set_gray_to_rgb( png_ptr );
        break;
    default:
        /* printf( "color_type = %u\n", color_type ); */
        png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
        fclose( fp );
        return 1;
    }
    if ( ~color_type & PNG_COLOR_MASK_ALPHA )
        png_set_filler( png_ptr, 0xff, PNG_FILLER_AFTER );
    png_read_update_info( png_ptr, info_ptr );
    *w = png_get_image_width( png_ptr, info_ptr );
    *h = png_get_image_height( png_ptr, info_ptr );
    bpl = *w << 2;
    if ( png_get_rowbytes( png_ptr, info_ptr ) == bpl )
        if ( *data = malloc( *h * bpl ) )
            if ( rows = malloc( sizeof( png_bytep ) * *h ) ) {
                for ( i = 0 ; i < *h ; i++ )
                    rows[ i ] = *data + i * bpl;
                png_read_image( png_ptr, rows );
                png_read_end( png_ptr, NULL );
                free( (void *)rows );
                res = 0;
            } else
                free( (void *)( *data ) );
    png_destroy_read_struct( &png_ptr, &info_ptr, NULL );
    fclose( fp );
    return res;
}

uns8b trp_pix_save_png( trp_obj_t *pix, trp_obj_t *path )
{
    FILE *fp;
    uns32b w, h, bpl, i;
    uns8b *cpath;
    png_structp png_ptr;
    png_infop info_ptr;
    png_bytep *rows = NULL;

    if ( pix->tipo != TRP_PIX )
        return 1;
    if ( ((trp_pix_t *)pix)->map.p == NULL )
        return 1;
    cpath = trp_csprint( path );
    fp = trp_fopen( cpath, "wb" );
    trp_csprint_free( cpath );
    if ( fp == NULL )
        return 1;
    if ( ( png_ptr = png_create_write_struct( PNG_LIBPNG_VER_STRING,
                                              NULL, NULL, NULL ) ) == NULL ) {
        fclose( fp );
        return 1;
    }
    if ( ( info_ptr = png_create_info_struct( png_ptr ) ) == NULL ) {
        png_destroy_write_struct( &png_ptr, NULL );
        fclose( fp );
        return 1;
    }
    if ( setjmp( png_jmpbuf( png_ptr ) ) ) {
        free( (void *)rows );
        png_destroy_write_struct( &png_ptr, &info_ptr );
        fclose( fp );
        return 1;
    }
    w = ((trp_pix_t *)pix)->w;
    h = ((trp_pix_t *)pix)->h;
    bpl = w << 2;
    if ( ( rows = malloc( sizeof( png_bytep ) * h ) ) == NULL ) {
        png_destroy_write_struct( &png_ptr, &info_ptr );
        fclose( fp );
        return 1;
    }
    for ( i = 0 ; i < h ; i++ )
        rows[ i ] = ((trp_pix_t *)pix)->map.p + i * bpl;
    png_init_io( png_ptr, fp );
    png_set_compression_level( png_ptr, Z_BEST_COMPRESSION );
    png_set_IHDR( png_ptr, info_ptr, w, h, 8,
                  PNG_COLOR_TYPE_RGB_ALPHA,
                  PNG_INTERLACE_NONE,
                  PNG_COMPRESSION_TYPE_DEFAULT,
                  PNG_FILTER_TYPE_DEFAULT );
    png_set_rows( png_ptr, info_ptr, rows );
    png_write_png( png_ptr, info_ptr, 0, NULL );
    free( (void *)rows );
    png_destroy_write_struct( &png_ptr, &info_ptr );
    fclose( fp );
    return 0;
}



